#version 330 core

// Input vertex data, different for all executions of this shader.
layout(location = 3) in vec3 squareVertices;
layout(location = 4) in vec4 xyzs; // Position of the center of the particule and size of the square
layout(location = 5) in vec4 incolor; // Position of the center of the particule and size of the square

// Output data ; will be interpolated for each fragment.
out vec2 fragUV;
out vec4 fragParticleColor;

// Values that stay constant for the whole mesh.
uniform vec3 varCameraRight;
uniform vec3 varCameraUp;
uniform mat4 varVP;

void main()
{
	float particleSize = xyzs.w; // because we encoded it this way.
	vec3 particleCenter = xyzs.xyz;

	vec3 vertexPosition = particleCenter
	+ varCameraRight * squareVertices.x * particleSize
	+ varCameraUp * squareVertices.y * particleSize;

	// Output position of the vertex
	gl_Position = varVP * vec4(vertexPosition, 1.0f);

	// UV of the vertex. No special space for this one.
	fragUV = squareVertices.xy + vec2(0.5, 0.5);
	fragParticleColor = incolor;
}

