#version 330 core

// Interpolated values from the vertex shaders
in vec2 fragUV;
in vec3 fragPosition;
in vec3 fragNormal;
in vec3 fragEyeDirection;
in vec3 fragLightDirection;

// Ouput data
out vec4 color;

// Values that stay constant for the whole mesh.
uniform sampler2D varLightTextureSampler;
uniform mat4 varMV;
uniform vec3 varLightPosition;

void main(){

	// Light emission properties
	// You probably want to put them as uniforms
	vec3 LightColor = vec3(1,1,1);
	float LightPower = 50.0f;
	
	// Material properties
	vec3 MaterialDiffuseColor = texture2D( varLightTextureSampler, fragUV ).rgb;
	float alpha = texture2D(varLightTextureSampler, fragUV).a;
	vec3 MaterialAmbientColor = vec3(0.1,0.1,0.1) * MaterialDiffuseColor;
	vec3 MaterialSpecularColor = vec3(0.3,0.3,0.3);

	// Distance to the light
	float distance = length( varLightPosition - fragPosition );

	// Normal of the computed fragment, in camera space
	vec3 n = normalize( fragNormal );
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize( fragLightDirection );
	// Cosine of the angle between the normal and the light direction, 
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendicular to the triangle -> 0
	//  - light is behind the triangle -> 0
	float cosTheta = clamp( dot( n,l ), 0,1 );
	
	// Eye vector (towards the camera)
	vec3 E = normalize(fragEyeDirection);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	// Cosine of the angle between the Eye vector and the Reflect vector,
	// clamped to 0
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	
	color = 
		// Ambient : simulates indirect lighting
		vec4(MaterialAmbientColor +
		// Diffuse : "color" of the object
		MaterialDiffuseColor * LightColor * LightPower * cosTheta / (distance*distance) +
		// Specular : reflective highlight, like a mirror
		MaterialSpecularColor * LightColor * LightPower * pow(cosAlpha,5) / (distance*distance), alpha);

}