#version 330 core

// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 inPosition;
layout(location = 1) in vec2 inUV;
layout(location = 2) in vec3 inNormal;

// Output data ; will be interpolated for each fragment.
out vec2 fragUV;
out vec3 fragPosition;
out vec3 fragNormal;
out vec3 fragEyeDirection;
out vec3 fragLightDirection;

// Values that stay constant for the whole mesh.
uniform mat4 varMVP;
uniform mat4 varV;
uniform mat4 varM;
uniform vec3 varLightPosition;

void main(){

	// Output position of the vertex, in clip space : MVP * position
	gl_Position =  varMVP * vec4(inPosition,1);
	
	// Position of the vertex, in worldspace : M * position
	fragPosition = (varM * vec4(inPosition,1)).xyz;
	
	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	vec3 tempOutPosition = ( varV * varM * vec4(inPosition,1)).xyz;
	fragEyeDirection = vec3(0,0,0) - tempOutPosition;

	// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity.
	vec3 tempLightPosition = ( varV * vec4(varLightPosition,1)).xyz;
	fragLightDirection = tempLightPosition + fragEyeDirection;
	
	// Normal of the the vertex, in camera space
	fragNormal = ( varV * varM * vec4(inNormal,0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	
	// UV of the vertex. No special space for this one.
	fragUV = inUV;
}

